package com.bpcl.lpg.core.payment.billdesk.impl;

import org.slf4j.LoggerFactory;
import java.text.MessageFormat;
import org.apache.http.util.EntityUtils;
import com.bpcl.lpg.core.model.EasysecsModel;
import java.util.Objects;
import com.bpcl.core.util.BPCLAmountFormatter;
import com.bpcl.core.util.BPCLDateUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.impl.client.CloseableHttpClient;
import java.io.IOException;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.HttpEntity;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.NameValuePair;
import java.util.ArrayList;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.config.SocketConfig;
import org.apache.http.client.config.RequestConfig;
import com.bpcl.lpg.core.enums.LpgPaymentStatusEnum;
import com.bpcl.lpg.core.utill.BillDeskMsgUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.bpcl.core.custom.exception.BpclInvalidPaymentResponseException;
import com.bpcl.facades.payment.refill.RefillBookingData;
import com.bpcl.lpg.core.pojo.LpgPaymentResponseData;
import org.apache.commons.lang3.StringUtils;
import com.bpcl.lpg.core.model.LpgPaymentInfoModel;
import com.bpcl.facades.payment.refund.RefundMsdData;
import com.bpcl.lpg.core.helper.LpgRefillHelper;
import javax.annotation.Resource;
import com.bpcl.lpg.core.service.FetchPaymentStatusService;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import com.bpcl.lpg.core.pojo.BillDeskResponseMsgData;
import java.util.List;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import com.bpcl.lpg.core.payment.billdesk.LPGPaymentService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.lpg.core.payment.billdesk.LPGChecksumService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.payment.billdesk.LPGRefundService;

public class LPGRefundServiceImpl implements LPGRefundService
{
    private static final Logger LOG;
    private static final int TIMEOUT = 30000;
    private LPGChecksumService lpgChecksumService;
    private ConfigurationService configurationService;
    private LPGPaymentService lpgPaymentService;
    private Converter<List<String>, BillDeskResponseMsgData> lpgRefundStatusResponseDataConverter;
    private LPGCredentialsService lpgCredentialsService;
    @Resource
    private FetchPaymentStatusService fetchPaymentStatusService;
    @Resource
    private LpgRefillHelper lpgRefillHelper;
    
    public void processRefund(final RefundMsdData refundMsdData, final LpgPaymentInfoModel paymentInfoModel) throws BpclInvalidPaymentResponseException, JsonProcessingException {
        final LpgPaymentResponseData responseData = this.fetchPaymentStatusService.fetchPaymentStatus(paymentInfoModel);
        final String transactionId = paymentInfoModel.getTransactionID();
        LPGRefundServiceImpl.LOG.info("Payment status for transaction {} is {} and customer :: {}", transactionId, responseData, paymentInfoModel.getCustomerId());
        if ("200".equals(responseData.getStatus_Code())) {
            final RefillBookingData refillBookingData = this.lpgRefillHelper.prepareRefillBookingData(responseData);
            this.lpgPaymentService.updateRefillStatusByTxnID(transactionId, refillBookingData);
            LPGRefundServiceImpl.LOG.info("Refill booking is success for the transaction:{} and customer :: {}", transactionId, paymentInfoModel.getCustomerId());
        }
        else if ("770".equals(responseData.getStatus_Code())) {
            final String refundMsg = this.prepareRefundMsg(refundMsdData);
            final String refundUrl = this.configurationService.getConfiguration().getString("billdesk.refund.url");
            final String refundQueryUrl = this.configurationService.getConfiguration().getString("billdesk.refund.query.url");
            if (LPGRefundServiceImpl.LOG.isDebugEnabled()) {
                LPGRefundServiceImpl.LOG.debug("Refund request message is : {}", refundMsg);
            }
            String refundResponseMsg = this.executePostWoAuthorization(refundUrl, refundMsg);
            if (StringUtils.isBlank(refundResponseMsg)) {
                refundResponseMsg = this.executePostWoAuthorization(refundQueryUrl, refundMsg);
            }
            this.updateRefundDataToPaymentInfo(refundMsdData, refundResponseMsg, paymentInfoModel);
        }
    }
    
    public void updateRefundDataToPaymentInfo(final RefundMsdData refundMsdData, final String refundResponseMsg, final LpgPaymentInfoModel paymentInfoModel) throws BpclInvalidPaymentResponseException {
        final String transactionID = refundMsdData.getTransactionID();
        if (this.lpgPaymentService.isAuthorizedResponse(refundResponseMsg)) {
            final List<String> refundParams = BillDeskMsgUtils.convertMsgToAttributes(refundResponseMsg, 14);
            final BillDeskResponseMsgData refundData = (BillDeskResponseMsgData)this.lpgRefundStatusResponseDataConverter.convert((Object)refundParams);
            refundData.setLastRefundRefNo(refundMsdData.getRefundRefNo());
            refundData.setCustomerID(transactionID);
            this.checkRefundErrorCodes(refundData, paymentInfoModel);
        }
    }
    
    public void checkRefundErrorCodes(final BillDeskResponseMsgData responseMsg, final LpgPaymentInfoModel lpgPaymentInfoModel) throws BpclInvalidPaymentResponseException {
        final String RefundErrorCode = responseMsg.getRefundErrorCode();
        if (this.isValidRefundStatus(responseMsg.getRefStatus())) {
            lpgPaymentInfoModel.setTransactionStatus(LpgPaymentStatusEnum.REFUND_SUCCESSFUL);
            lpgPaymentInfoModel.setIsRefunded(Boolean.valueOf(true));
        }
        else if (StringUtils.equalsIgnoreCase("ERR_REF009", RefundErrorCode) || StringUtils.equalsIgnoreCase("ERR_REF010", RefundErrorCode)) {
            lpgPaymentInfoModel.setTransactionStatus(LpgPaymentStatusEnum.REFUND_SUCCESSFUL);
            lpgPaymentInfoModel.setIsRefunded(Boolean.valueOf(true));
        }
        else if (StringUtils.isNotBlank(RefundErrorCode) && !StringUtils.equalsIgnoreCase("ERR_REF016", RefundErrorCode) && !this.IsValidException(RefundErrorCode)) {
            LPGRefundServiceImpl.LOG.error("Refund Response received as Error/Exception from Billdesk : {} for transactions ID : {}", RefundErrorCode, lpgPaymentInfoModel.getTransactionID());
            lpgPaymentInfoModel.setTransactionStatus(LpgPaymentStatusEnum.ERROR);
        }
        this.lpgPaymentService.updateRefundDataToPaymentInfo(responseMsg, lpgPaymentInfoModel);
    }
    
    private boolean isValidRefundStatus(final String refundStatus) {
        return StringUtils.equalsIgnoreCase(refundStatus, "0699") || StringUtils.equalsIgnoreCase(refundStatus, "0799") || StringUtils.equalsIgnoreCase(refundStatus, "0899");
    }
    
    private boolean IsValidException(final String refundErrorCode) {
        return StringUtils.equalsIgnoreCase(refundErrorCode, "ERREXCEPTION1001") || StringUtils.equalsIgnoreCase(refundErrorCode, "ERREXCEPTION1002") || StringUtils.equalsIgnoreCase(refundErrorCode, "ERREXCEPTION1003") || StringUtils.equalsIgnoreCase(refundErrorCode, "ERREXCEPTION1004");
    }
    
    public String executePostWoAuthorization(final String uri, final String msg) {
        final RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(30000).setSocketTimeout(30000).build();
        final SocketConfig socketConfig = SocketConfig.custom().setSoTimeout(30000).build();
        final CloseableHttpClient client = HttpClients.custom().setDefaultRequestConfig(requestConfig).setDefaultSocketConfig(socketConfig).build();
        final HttpPost post = new HttpPost(uri);
        final List<NameValuePair> arguments = new ArrayList<NameValuePair>();
        arguments.add(new BasicNameValuePair("msg", msg));
        try {
            post.setEntity(new UrlEncodedFormEntity(arguments));
            final CloseableHttpResponse response = client.execute((HttpUriRequest)post);
            return this.getResponseMsg(response);
        }
        catch (final IOException exception) {
            LPGRefundServiceImpl.LOG.error("Exception while executing the query status check controller", exception.getCause());
            return "";
        }
    }
    
    public String prepareRefundMsg(final RefundMsdData refundMsdData) {
        final String txnDate = BPCLDateUtils.convertSrcToTargetPattern(refundMsdData.getTxnDate(), "dd-MM-yyyy HH:mm:ss", "yyyyMMdd");
        final String txnAmount = BPCLAmountFormatter.formatAmount(refundMsdData.getTxnAmount());
        final EasysecsModel merchantIDModel = this.lpgCredentialsService.getSecretsByKey("billdeskMerchantId");
        Objects.requireNonNull(merchantIDModel.getSecret(), "Merchant id should not be null");
        final String[] msgParams = { "0400", merchantIDModel.getSecret(), refundMsdData.getBilldeskTxnID(), txnDate, refundMsdData.getTransactionID(), "INR", txnAmount, txnAmount, BPCLDateUtils.getCurrentDateAsString("yyyyMMddHHmmss"), refundMsdData.getRefundRefNo(), "NA", "NA", "NA" };
        final String msgWoChecksum = StringUtils.join((Object[])msgParams, "|");
        final String checksumValue = this.lpgChecksumService.getChecksumValue(msgWoChecksum);
        return msgWoChecksum + "|" + checksumValue;
    }
    
    private String getResponseMsg(final CloseableHttpResponse response) throws IOException {
        final String body = (response == null) ? null : EntityUtils.toString(response.getEntity());
        if (LPGRefundServiceImpl.LOG.isInfoEnabled()) {
            LPGRefundServiceImpl.LOG.info(MessageFormat.format("Response from bill desk for Refund {0} ", body));
        }
        return body;
    }
    
    public void setLpgCredentialsService(final LPGCredentialsService lpgCredentialsService) {
        this.lpgCredentialsService = lpgCredentialsService;
    }
    
    public void setLpgChecksumService(final LPGChecksumService lpgChecksumService) {
        this.lpgChecksumService = lpgChecksumService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public void setLpgPaymentService(final LPGPaymentService lpgPaymentService) {
        this.lpgPaymentService = lpgPaymentService;
    }
    
    public void setLpgRefundStatusResponseDataConverter(final Converter<List<String>, BillDeskResponseMsgData> lpgRefundStatusResponseDataConverter) {
        this.lpgRefundStatusResponseDataConverter = lpgRefundStatusResponseDataConverter;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LPGRefundServiceImpl.class);
    }
}
